# Copyright (C) 2005 Matthew Good <trac@matt-good.net>
# Copyright (C) 2005 Jan Finell <finell@cenix-bioscience.com> 
#
# "THE BEER-WARE LICENSE" (Revision 42):
# <trac@matt-good.net> wrote this file.  As long as you retain this notice you
# can do whatever you want with this stuff.  If we meet some day, and you think
# this stuff is worth it, you can buy me a beer in return.  Matthew Good
# (Beer-ware license written by Poul-Henning Kamp
#  http://people.freebsd.org/~phk/)
#
# Author: Matthew Good <trac@matt-good.net>
# Month/Year navigation by: Jan Finell <finell@cenix-bioscience.com>
# Month/Year increment/decrement parameters by: Tony Abou-Assaleh <taa@acm.org>

import time
import calendar
from cStringIO import StringIO
from trac.wiki.api import WikiSystem
from trac.util import *

# format:
#   WikiCalendar([year,month,[showbuttons,[wiki_page_format]]])
#
#   displays a calendar, the days link to:
#     - milestones (day in bold) if there is one on that day
#     - a wiki page that has wiki_page_format (if exist)
#     - create that wiki page if it does not exist
#
# arguments: 
#   year, month = display calendar for month in year ('*' for current year/month)
#   showbuttons = true/false, show prev/next buttons
#   wiki_page_format = strftime format for wiki pages to display as link
#                      (if there is not a milestone placed on that day)
#                      (if exist, otherwise link to create page)
#                      default is "%Y-%m-%d"
#
# examples:
#     WikiCalendar(2006,07)
#     WikiCalendar(2006,07,false)
#     WikiCalendar(*,*,true,Meeting-%Y-%m-%d)
#     WikiCalendar(2006,07,false,Meeting-%Y-%m-%d)

def execute(hdf, txt, env):
    today = time.localtime()
    http_param_year = hdf.getValue("args.year", "")
    http_param_month = hdf.getValue("args.month", "")
    if txt:
        args = txt.split(',', 3)
    else:
        args = []
    
    # find out whether use http param, current or macro param year/month
    
    if http_param_year == "":
        # use current year
        year = today.tm_year
    else:
        # year in http params (clicked by user) overrides everything
        year = int(http_param_year)

    # not clicked on a prev or next button
    if len(args) >= 1:
	if args[0][0] == "*":
		pass
	elif args[0][0] in ("-", "+"):
		year += int(args[0])
	else:
        	# year given in macro parameters
        	year = int(args[0])

    if http_param_month == "":
        # use current month
        month = today.tm_mon
    else:
        # month in http params (clicked by user) overrides everything
        month = int(http_param_month)

    # not clicked on a prev or next button
    if len(args) >= 2:
	if args[1][1] == "*":
		pass
	elif args[1][1] in ("-", "+"):
		month += int(args[1])
	else:
        	# month given in macro parameters
        	month = int(args[1])

    # handle month > 12 and month < 1, must shift to month 
    # range from [1,12] to [0,11] and back
    year += (month-1) / 12
    month = (month+11) % 12 + 1

    showbuttons = 1
    if len(args) >= 3:
        showbuttons = bool(args[2]=="True" or args[2]=="true" or args[2]=="no" or args[2]=="0")
        
    wiki_page_format = "%Y-%m-%d"
    if len(args) >= 4:
        wiki_page_format = args[3]
    
    curr_day = None
    if year == today.tm_year and month == today.tm_mon:
        curr_day = today.tm_mday

    # Can use this to change the day the week starts on, but this
    # is a system-wide setting
    #calendar.setfirstweekday(calendar.SUNDAY)
    cal = calendar.monthcalendar(year, month)


    date = [year, month] + [1] * 7

    # url to the current page (used in the navigation links)
    thispageURL = env.href.wiki(hdf.getValue('wiki.page_name', ''))
    # for the prev/next navigation links
    prevMonth = month-1
    prevYear  = year
    nextMonth = month+1
    nextYear  = year
    # check for year change (KISS version)
    if prevMonth == 0:
        prevMonth = 12
        prevYear -= 1
    if nextMonth == 13:
        nextMonth = 1
        nextYear += 1

    # building the output
    buff = StringIO()
    buff.write('<table><caption>')

    if showbuttons:
        # prev month link
        prevMonthURL = thispageURL+'?month=%d&year=%d' % (prevMonth, prevYear)
        buff.write('<a href="%s">&lt; </a>' % prevMonthURL)
        
    # the caption
    buff.write(time.strftime('%B %Y', tuple(date)))
    
    if showbuttons:
        # next month link
        nextMonthURL = thispageURL+'?month=%d&year=%d' % (nextMonth, nextYear)
        buff.write('<a href="%s"> &gt;</a>' % nextMonthURL)
        
    buff.write('</caption>\n<thead><tr align="center">')
    
    for day in calendar.weekheader(2).split():
        buff.write('<th scope="col"><b>%s</b></th>' % day)
    buff.write('</tr></thead>\n<tbody>\n')

    for row in cal:
        buff.write('<tr align="right">')
        for day in row:
            if not day:
                buff.write('<td>&nbsp;</td>')
            else:
                # first check for milestone on that day
                db = env.get_db_cnx()
                cursor = db.cursor()
                # ugly way but i don't know how to set up a struct_time
                # (date as single numbers -> string -> time string parser -> struct_time -> mktime -> seconds
                duedateString = str(day) + "." + str(month) + "." + str(year)
                duedateStruct = time.strptime(duedateString, "%d.%m.%Y")
                duedate = time.mktime(duedateStruct)
                cursor.execute("SELECT name,due FROM milestone WHERE due=%s", (duedate,))
                row = cursor.fetchone()
                if row:
                    # found a milestone
                    milestone_name = row[0]
                    url = env.href.milestone(milestone_name)
                    buff.write('<td%(style)s><b><a href="%(url)s" alt="%(alt)s">%(day)s</a></b></td>' % {
                            'url': url,
                            'alt': milestone_name,
                            'day': day,
                            'style': day == curr_day and ' style="border: 1px solid #b00;"' or '',
                            })
                else:
                    # secondly check for wikipage with name specified in 'wiki_page_format'
                    date[2] = day
                    wiki = time.strftime(wiki_page_format, tuple(date))
                    url = env.href.wiki(wiki)
                    if WikiSystem(env).has_page(wiki):
                        buff.write('<td%(style)s><a href="%(url)s">%(day)s</a></td>' % {
                            'url': url,
                            'day': day,
                            'style': day == curr_day and ' style="border: 1px solid #b00;"' or '',
                            })
                    # just display the date
                    else:
                        url += "?action=edit"
                        buff.write('<td%(style)s><a href="%(url)s" class="missing">%(day)s</a></td>' % {
                            'url': url,
                            'day': day,
                            'style': day == curr_day and ' style="border: 1px solid #b00;"' or '',
                            })

        buff.write('</tr>\n')

    buff.write('</tbody>\n</table>')
            
    table = buff.getvalue()
    buff.close()
    return table

